/*******************************************************************************
 * Filename: SiteContextSearchBox.cs
 * Description: SiteContextSearchBox Web Control
 * Copyright: Copyright (C) Kunitaka / Unicorn Staff / Higurashi Creative Factory  All rights reserved.
 * Modified: Kunitaka / SharePoint Factory
 * Update History:
 * 0.01 2006.02.25 {ꉻΉJn
 * 0.10 2006.02.25 TvE[X
 ******************************************************************************/

using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Web.UI.HtmlControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.Utilities;

namespace Contoso.SharePoint.WebControls
{
	/// <summary>
	/// Summary description for SiteContextSearchBox.
	/// </summary>
	[ToolboxData("<{0}:SiteContextSearchBox runat=server></{0}:SiteContextSearchBox>")]
	[GuidAttribute("D0B4E209-69FE-48a3-AEFB-D087F424EFBE")]
	public class SiteContextSearchBox : SPControl, INamingContainer
	{
		// Variables to maintain five column table structure
		private TableCell _cellOne;
		private TableCell _cellTwo;
		private TableCell _cellThree;
		private TableCell _cellFour;
		private TableCell _cellFive;

		// Current site context
		private SPWeb _currentWeb;

		// Reference to original site search box - used when Portal is not available
		private Microsoft.SharePoint.WebControls.ViewSearchForm viewSearchForm;

/***** Kunitaka 2006.02.25 Add *****/
		const string strAdvanced = "xȌ";
		const string strExecute = "s܂";
		const string strKeyword = "̃TCgSPS(|[^)Ō";

		const string strAll = "ׂẴ\[X";
		const string strRoot = "`[TCgS";
		const string strSite = "̃TCg";

		const string strError = "P 1 ȏ͂ĂB";

		const string strTeams = "̃TCg";
		const string strGo = "";

		/// <summary> 
		/// Render this control to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		protected override void Render(HtmlTextWriter output)
		{
			EnsureChildControls();
			
			// The appropriate child control will render based on what was added to the collection
			try
			{
				RenderChildren(output);
			}
			catch(System.Exception ex)
			{
				output.WriteLine("The following error occurred in this Web Part.  Please contact your administrator for further assistance. \n" + ex.Message);
			}
		}

		protected void RenderSearchOptions()
		{
			// NOTE: If accessing the site anonymously at first, then authenticated, you need to ensure
			//		 the appropriate portal access settings.  
			//	Go to Site Settings, SharePoint Portal Server Central Administration,
			//	Configure alternate portal site URLs for intranet, extranet, and custom access
			//	and make sure the anonymous Intranet site is setup to map to the authenticated Intranet site

			// Set the parameters for site context and portal search page
			string allSourcesUrl="";
			string siteContextWhere="";
			string rootSiteContextWhere="";

			// Construct the All Sources query
			allSourcesUrl = _currentWeb.PortalUrl + "search.aspx?k=";

			//Where Clause to construct:
			//	CONTAINS("DAV:href",'"[ServerRelativeUrl]"')AND("DAV:href"LIKE'%[ServerRelativeUrl]/%')
			//We are combining CONTAINS and LIKE clauses for speed and accuracy.  
			//The CONTAINS clause ignores "/" characters so there is no way to exclude sibling sites from the results.  
			//The LIKE clause honors the traling "/" but by itself would take too long to execute.

			// Construct the Site Context query
			siteContextWhere =  "&w="
				// Url encode our query string 
				+ SPEncode.UrlEncode("CONTAINS(\"DAV:href\",'\"" 
				// SQL encode our where clause
				+ _currentWeb.ServerRelativeUrl.Replace("'","''") 
				+ "\"')AND(\"DAV:href\"LIKE'%"
				//SQL encode our where clause
				+ _currentWeb.ServerRelativeUrl.Replace("'","''") 
				+ "/%')");

			// Construct the Root Web query, if necessary
			if(!_currentWeb.IsRootWeb)
			{
				rootSiteContextWhere = "&w="
					// Url encode our query string
					+ SPEncode.UrlEncode("CONTAINS(\"DAV:href\",'\"" 
					//SQL encode our where clause
					+ _currentWeb.Site.RootWeb.ServerRelativeUrl.Replace("'","''") 
					+ "\"')AND(\"DAV:href\"LIKE'%"
					//SQL encode our where clause
					+ _currentWeb.Site.RootWeb.ServerRelativeUrl.Replace("'","''") 
					+ "/%')");
			}

			// Build the Client Side search function...
			StringBuilder searchPortal = new StringBuilder();
			searchPortal.Append("<SCRIPT language=JavaScript>\n");
			searchPortal.Append("<!--\n");
			// Determine the search keyword
			searchPortal.Append("function SubmitPortalSearch(arg, scopesId, searchStringId) {\n");
			searchPortal.Append("  var searchKeyword = document.getElementById(searchStringId).value;\n");
			searchPortal.Append("  if(searchKeyword == '')\n");
			searchPortal.Append("  {");
/***** Kunitaka 2006.02.25 Modified *****/
//			searchPortal.Append("    alert('Please enter one or more search words.');\n");
			searchPortal.Append("    alert('" + strError + "');\n");
			searchPortal.Append("		 if(null != event) event.returnValue = false;\n");
			searchPortal.Append("		 return;\n");
			searchPortal.Append("	 }\n");
			// Build base search string
			searchPortal.Append("	var searchUrl = \"" + allSourcesUrl + "\" + escape(searchKeyword);\n");
			// append advanced if clicked
			searchPortal.Append(" if(arg=='advanced') searchUrl+='&pt=all';\n");
			// append where clause if option selected from drop down
			searchPortal.Append("  var dd = document.getElementById(scopesId);\n");
			searchPortal.Append("  var scope = dd.options[dd.selectedIndex].value;\n");
			searchPortal.Append("  switch(scope){\n");
			searchPortal.Append("    case 'ALL':\n break;\n");
			searchPortal.Append("    case 'THIS':\n searchUrl+=\"" + SPEncode.ScriptEncode(siteContextWhere) + "\";\n break;\n");
			searchPortal.Append("    case 'ROOT':\n searchUrl+=\"" + SPEncode.ScriptEncode(rootSiteContextWhere) + "\";\n break;\n");
			searchPortal.Append("    default:break;}\n");
			searchPortal.Append("	window.top.window.location = searchUrl;\n");
			searchPortal.Append(" if(null != event) event.returnValue = false;\n");
			searchPortal.Append(" return;\n");
			searchPortal.Append("}\n");
			// handle Enter key if in a page that already had a form
			searchPortal.Append(" function CustomSearchKeyDown(event, scopesId, searchStringId){\n");
			searchPortal.Append(" if (IsKeyDownSubmit(event)){\n");
			searchPortal.Append("		SubmitPortalSearch('form',scopesId,searchStringId);\n");
			searchPortal.Append("		return false;}\n");
			searchPortal.Append("		return true;}\n");
			searchPortal.Append("// -->\n");
			searchPortal.Append("</SCRIPT>");

			//  Only register the script once
			if(!Page.IsClientScriptBlockRegistered("searchPortal"))
				Page.RegisterClientScriptBlock("searchPortal", searchPortal.ToString());
		}


		protected void RenderSPSFunctions()
		{
			// Image hover function and image preload (directly from ViewSource of search.aspx)...
			StringBuilder imageHover = new StringBuilder();

			imageHover.Append("<SCRIPT language=JavaScript>\n");
			imageHover.Append("<!--\n");
			imageHover.Append("function newImage(arg) {\n");
			imageHover.Append("	var	rslt = new Image();\n");
			imageHover.Append("	rslt.src = arg;\n");
			imageHover.Append("	return rslt;\n");
			imageHover.Append("}\n");
			imageHover.Append("function changeImages(a, b)	{\n");
			imageHover.Append("	a.src =	b;\n");
			imageHover.Append("}\n");
			imageHover.Append("newImage(\"\\/_layouts\\/images\\/icongo02.gif\");\n");
			imageHover.Append("newImage(\"\\/_layouts\\/images\\/icongo03.gif\");\n");
			imageHover.Append("// -->\n");
			imageHover.Append("</SCRIPT>");

			// Only register the script once
			if(!Page.IsClientScriptBlockRegistered("imageHover"))
				Page.RegisterClientScriptBlock("imageHover", imageHover.ToString());
		}


		protected override void CreateChildControls()
		{
			_currentWeb = SPControl.GetContextWeb(Context);
			
			if(_currentWeb.PortalMember) // && System.Threading.Thread.CurrentPrincipal.Identity.IsAuthenticated)
			{
				// We need to render the SPS functions we want to use
				RenderSPSFunctions();

				// We need to render our client-side code that handles submitting the search
				RenderSearchOptions();

				#region Column 1 - spacer
				// Create first column
				// <td class=\"ms-searchform\" noWrap colSpan=2 height=32 style=\"padding-left:	4px\"></td>
				// Simple spacer
				_cellOne = new TableCell();
/***** Kunitaka 2006.02.25 Modified *****/
//				_cellOne.CssClass = "ms-searchform";
				_cellOne.CssClass = "ms-searchcorner";
				_cellOne.Wrap = false;
				_cellOne.Height = Unit.Pixel(32);
/***** Kunitaka 2006.02.25 Modified *****/
//				_cellOne.Attributes.Add("style", "padding-left: 4px;");
				_cellOne.Attributes.Add("style", "padding-left: 6px;");
				Controls.Add(_cellOne);
				#endregion

				#region Column 2 - Advanced Search image button
				// Create second column
				// <td class=\"ms-searchform\" nowrap=\"\">
				// <a accesskey=\"S\" title=\"Advanced Search\" href=\"\">
				// <img title=\"Advanced Search\" alt=\"Advanced Search\" src=\"/_layouts/images/SPSSearch2.gif\" 
				//	border=\"0\" style=\"vertical-align: 2;cursor:hand;\" />
				// </a>
				// </td>
				_cellTwo = new TableCell();
				_cellTwo.CssClass = "ms-searchform";
				_cellTwo.Wrap = false;
				Controls.Add(_cellTwo);
				// Advanced Search magnifying glass
				HyperLink advanced = new HyperLink();
				advanced.AccessKey = "S";
/***** Kunitaka 2006.02.25 Modified *****/
//				advanced.ToolTip = "Advanced Search";
//				advanced.Text = "Advanced Search";
				advanced.ToolTip = strAdvanced;
				advanced.Text = strAdvanced;
				//advanced.NavigateUrl --> added after all controls are in collection to obtain proper ClientID
				advanced.ImageUrl = "/_layouts/images/SPSSearch2.gif";
				advanced.BorderWidth = 0;
				advanced.Attributes.Add("style", "vertical-align: 2;cursor:hand;");
				_cellTwo.Controls.Add(advanced);
				#endregion

				#region Column 3 - Scopes dropdown and Search Keyword textbox
				// Create third column
				// <td class=\"ms-searchform\" style=\"padding-left:	4px\">
				// <select name=\"RightBodySectionSearchBoxID:SBScopesDDL\" id=\"RightBodySectionSearchBoxID_SBScopesDDL\" 
				//	class=\"ms-descriptiontext\" Title=\"Select a scope\">
				// 	<option selected=\"selected\" value=\"S_6590072E2A\">All sources</option>
				// 	<option value=\"S_847C885313\">Test</option>
				// </select>
				// <input name=\"RightBodySectionSearchBoxID:SCB6C0B0A_InputKeywords\" 
				//	type=\"text\" maxlength=\"300\" id=\"RightBodySectionSearchBoxID_SCB6C0B0A_InputKeywords\" 
				//	accesskey=\"s\" title=\"Enter keyword(s)\" class=\"ms-searchbox ms-descriptiontext\" 
				//	alt=\"Enter keyword(s)\" onkeypress=\"javascript:OSBKDCS(event, 'SCB6C0B0A_');\" style=\"width:170px;\" />
				// </td>
				// WORKAROUND NOTE: 
				//	The WebPart Framework will not allow us to add a simple HtmlForm control like standard ASP.NET
				//	does.  The following 2 lines are the preferred method, but unusable in a Web Part Page.
				//				HtmlForm form = new HtmlForm();
				//				form.ID = "frmSiteSearch";
				//	Therefore, a literal control with the Form tag is rendered when appropriate
				_cellThree = new TableCell();
				_cellThree.CssClass = "ms-searchform";
				_cellThree.Wrap = false;
				_cellThree.Attributes.Add("style","padding-left: 4px;");
				Controls.Add(_cellThree);
				Control parent = this.Parent;
				// Add Scopes drop down
				HtmlSelect scopes = new HtmlSelect();
				scopes.ID = scopes.ClientID;
				scopes.Attributes.Add("class", "ms-descriptiontext");
/***** Kunitaka 2006.02.25 Modified *****/
//				scopes.Attributes.Add("style", "vertical-align: 3");
				scopes.Attributes.Add("style", "vertical-align: 2");
				ListItem item;
/***** Kunitaka 2006.02.25 Modified *****/
//				item = new ListItem("All Sources", "ALL");
				item = new ListItem(strAll, "ALL");
				scopes.Items.Add(item);
				if(!_currentWeb.IsRootWeb)
				{
/***** Kunitaka 2006.02.25 Modified *****/
//					item = new ListItem("Site Collection", "ROOT");
					item = new ListItem(strRoot, "ROOT");
					scopes.Items.Add(item);
				}
/***** Kunitaka 2006.02.25 Modified *****/
//				item = new ListItem("This Site", "THIS");
				item = new ListItem(strSite, "THIS");
				item.Selected = true;
				scopes.Items.Add(item);
				// Add search keyword text box
				HtmlInputText searchString = new HtmlInputText();
				searchString.ID = searchString.ClientID;
				searchString.Size = 25;
				searchString.MaxLength = 300;
				searchString.Attributes.Add("accesskey", "S");
				searchString.Attributes.Add("class", "ms-searchbox ms-descriptiontext");
/***** Kunitaka 2006.02.25 Modified *****/
//				searchString.Attributes.Add("style", "width:170px;vertical-align: 3");
				searchString.Attributes.Add("style", "width:170px;vertical-align: 2");
/***** Kunitaka 2006.02.25 Modified *****/
//				searchString.Attributes.Add("alt", "Search this Site via the Portal");
//				searchString.Attributes.Add("title", "Search this Site via the Portal");
				searchString.Attributes.Add("alt", strKeyword);
				searchString.Attributes.Add("title", strKeyword);

				// Crawl the control tree to see if we live in a form
				// If we don't have a form anywhere, we will create our own to handle submit
				while ((parent != null) &&	!(parent is HtmlForm))
				{
					parent = parent.Parent;
				}
				bool emitForm = (parent == null);
			
				HtmlGenericControl form = new HtmlGenericControl("form");
				if(emitForm)
				{
					form.ID = "mcsCustomSiteSearch";
					form.Attributes.Add("method", "GET");
					//form.Attributes.Add("action",...) --> added after all controls are in collection to obtain proper ClientID
					_cellThree.Controls.Add(form);
					form.Controls.Add(scopes);
					form.Controls.Add(searchString);
				}
				else
				{
					_cellThree.Controls.Add(scopes);
					//searchString.Attributes.Add("onKeyDown",...) --> added after all controls are in collection to obtain proper ClientID
					_cellThree.Controls.Add(searchString);
				}
				#endregion

				#region Column 4 - Go image button
				// Create fourth column
				// <td class=\"ms-searchform\" nowrap=\"\" style=\"padding-left:	4px\">
				// <a accesskey=\"s\" title=\"Execute search\" href=\"javascript:SCB6C0B0A_OGBCLS(true)\">
				// <img id=\"RightBodySectionSearchBoxID_SCB6C0B0A_go\" title=\"Execute search\" alt=\"Execute search\" 
				//	border=\"0\" ONMOUSEOVER=\"changeImages(this, '\\/_layouts\\/images\\/icongo02.gif'); return true;\" 
				//	ONMOUSEOUT=\"changeImages(this,	'\\/_layouts\\/images\\/icongo01.gif'); return	true;\" 
				//	ONMOUSEDOWN=\"changeImages(this, '\\/_layouts\\/images\\/icongo03.gif'); return true;\" 
				//	ONMOUSEUP=\"changeImages(this, '\\/_layouts\\/images\\/icongo02.gif');	return true;\" 
				//	src=\"/_layouts/images/icongo01.gif\" border=\"0\" style=\"vertical-align: 2;cursor:hand;\" /></a>
				// <img src='/_layouts/images/blank.gif' width=4px alt=''/>
				// </td>
				_cellFour = new TableCell();
				_cellFour.CssClass = "ms-searchform";
				_cellFour.Wrap = false;
				_cellFour.Attributes.Add("style","padding-left: 4px;");
				Controls.Add(_cellFour);
				// Add Hyperlink, then add image to hyperlink
				HyperLink go = new HyperLink();
				go.AccessKey = "S";
/***** Kunitaka 2006.02.25 Modified *****/
//				go.ToolTip = "Execute Search";
//				go.Text = "Execute Search";
				go.ToolTip = strExecute;
				go.Text = strExecute;
				//go.NavigateUrl --> added after all controls are in collection to obtain proper ClientID
				_cellFour.Controls.Add(go);
				// Add the image control
				Image imageGo = new Image();
/***** Kunitaka 2006.02.25 Modified *****/
//				imageGo.AlternateText = "Execute Search";
				imageGo.AlternateText = strExecute;
				imageGo.ImageUrl = "/_layouts/images/icongo01.gif";
				imageGo.BorderWidth = 0;
				imageGo.Attributes.Add("ONMOUSEOVER", "changeImages(this, '\\/_layouts\\/images\\/icongo02.gif'); return true;");
				imageGo.Attributes.Add("ONMOUSEOUT", "changeImages(this, '\\/_layouts\\/images\\/icongo01.gif'); return true;");
				imageGo.Attributes.Add("ONMOUSEDOWN", "changeImages(this, '\\/_layouts\\/images\\/icongo03.gif'); return true;");
				imageGo.Attributes.Add("ONMOUSEUP", "changeImages(this, '\\/_layouts\\/images\\/icongo02.gif'); return true;");
				imageGo.Attributes.Add("style", "vertical-align: 2;cursor:hand;");
				go.Controls.Add(imageGo);
				// Add a spacer image
				Image imageBlank = new Image();
				imageBlank.ImageUrl = "/_layouts/images/blank.gif";
				imageBlank.Width = Unit.Pixel(4);
				imageBlank.AlternateText = "";
				_cellFour.Controls.Add(imageBlank);
				#endregion

				#region Column 5 - spacer
				// Create fifth column
				// <td class=\"ms-searchform\" noWrap colSpan=2 height=32 style=\"padding-left:	4px\"></td>
				// Simple spacer column
				_cellFive = new TableCell();
				_cellFive.CssClass = "ms-searchform";
				_cellFive.Wrap = false;
				_cellFive.ColumnSpan = 2;
				_cellFive.Height = Unit.Pixel(32);
				_cellFive.Attributes.Add("style","padding-left: 4px");
				Controls.Add(_cellFive);
				#endregion

				#region Wire Client-Side events
				// Now that all controls have been added to the Controls collection, they have the
				// ClientID values we needed for our Client-Side scripts
				HyperLink tempAdvancedSearch = (HyperLink)_cellTwo.Controls[0];
				tempAdvancedSearch.NavigateUrl = "javascript:SubmitPortalSearch('advanced', '" + SPEncode.ScriptEncode(scopes.ClientID) + "', '" + SPEncode.ScriptEncode(searchString.ClientID) + "');"; 
				HyperLink tempGo = (HyperLink)_cellFour.Controls[0];
				tempGo.NavigateUrl = "javascript:SubmitPortalSearch('go', '" + SPEncode.ScriptEncode(scopes.ClientID) + "', '" + SPEncode.ScriptEncode(searchString.ClientID) + "');"; 

				// If we emit our own form, then we simply hook the Submit function to the form action since 
				// pressing Enter in the text box will cause the form to submit
				if(emitForm)
				{
					HtmlGenericControl tempForm = (HtmlGenericControl)_cellThree.Controls[0];
					tempForm.Attributes.Add("action", "javascript:SubmitPortalSearch('form', '" + SPEncode.ScriptEncode(scopes.ClientID) + "', '" + SPEncode.ScriptEncode(searchString.ClientID) + "');"); 
				}
				// However, if we are on a page that already has a form, we need to hook the Submit to the Enter
				// keypress by intercepting the press event
				else
				{	
					HtmlInputText tempText = (HtmlInputText)_cellThree.Controls[1];
					tempText.Attributes.Add("onKeyDown", "return CustomSearchKeyDown(event, '" + SPEncode.ScriptEncode(scopes.ClientID) + "','" + SPEncode.ScriptEncode(searchString.ClientID) + "');");
				}
				#endregion
			}
			else
			{
				// TODO: For full fidelity, the following properties should be implemented as properties on our search box
				//<SharePoint:ViewSearchForm ID="L_SearchView" Prompt="Search this site" Go="Go" Action="searchresults.aspx" runat="server"/>
				viewSearchForm = new ViewSearchForm();
				viewSearchForm.ID = "L_SearchView";
/***** Kunitaka 2006.02.25 Modified *****/
//				viewSearchForm.Prompt = "Search this site";
//				viewSearchForm.Go = "Go";
				viewSearchForm.Prompt = strTeams;
				viewSearchForm.Go = strGo;
				viewSearchForm.Action = "searchresults.aspx";
				Controls.Add(viewSearchForm);
			}

			base.CreateChildControls();
		}
	}
}
